" Fix for Distributed Feature

  This file in fixes two problems:
  1. Sometimes, when a Remote Transcript is requested to be 
     opened it doesn't open.  This is especially true after a 
     CommonWidgets reinitialize.

   2. Attempting to browse a specific class, application, sender
      or implementor causes the prompt list to appear on the 
      server and the browser never opens.  This results in a 
      debug window."

DtDistributedUI becomeDefault !

!DtDistributedUI class publicMethods !

senders: anObjectSpace
		"Open a methods browser on the senders of a message"

	System showBusyCursorWhile: [
		| methodNames selectors methods browser |
		methodNames := System prompt: (self getMRIString: DtStringSendersOf group: #DsMessages).
			"Senders of?"
		(methodNames isNil or: [methodNames isEmpty]) ifFalse: [
			selectors :=  self findSelectorsInSpace: anObjectSpace using: methodNames. 
			selectors isEmpty ifFalse: [
				methods := (self dsInSpace: anObjectSpace) allMethodsSendingAll: selectors.
				methods isEmpty 
					ifTrue: [(self getMRI: DtMsgNoMethodsFound group: #DsMessages) display
						"No methods found"]
					ifFalse: [
						selectors size == 1
							ifTrue: [
								(browser := DtDistributedUI remoteBrowser: #highlightingMethods) == nil ifFalse: [
									(browser 
										on: methods 
												labeled: ((self getMRIString: DtTitleSendersOf group: #DsMessages) bindWith: selectors first printString)
															"Senders of %1" 
												highlighting: selectors first printString)
										associateObjectSpace: anObjectSpace;
"										labeled: ((self getMRIString: DtTitleSendersOf group: #DsMessages) bindWith: selectors first printString);
											""Senders of %1""
										highlighting: selectors first printString;
"										open].
								]
							ifFalse: [
								(browser := DtDistributedUI remoteBrowser: #methods) == nil ifFalse: [
									(browser
											on: methods labeled: ((self getMRIString: DtTitleSendersOfAll group: #DsMessages) bindWith: selectors asArray printString)
											"Senders of all of %1") 
										associateObjectSpace: anObjectSpace;
										open].
								]
						]
				]
			]
		]!

openHierarchyBrowser: anObjectSpace
	"Open a browser on the class provided by the user."

	(DtDistributedUI remoteBrowser: #classes) == nil ifTrue: [^self].
	System showBusyCursorWhile: [
		| theClass |
		theClass := self
			prompt: (self getMRIString: DtStringClassQ group: #DsMessages) "Class?"
			confirm: (self getMRIString: DtStringNotAClassTryAgain group: #DsMessages) 
				"'%1' is not the name of a class.\\Try again?" 
			answer: ''  "$NON-NLS$"
			convertUsing: [:className | self findClassInSpace: anObjectSpace using: className ifCancel: [self]].
		(theClass isNil or: [theClass == self]) ifFalse: [
			((DtDistributedUI remoteBrowser: #classes) on: theClass in: theClass controller)
				associateObjectSpace: anObjectSpace;
				open.
			]]!

openCategoryBrowser: anObjectSpace
	"Open a browser on a category provided by the user."

	(DtDistributedUI remoteBrowser: #methods) == nil ifTrue: [^self].
	System showBusyCursorWhile: [
		| theCategories |
		theCategories := self
			prompt: (self getMRIString: DtStringCategoryQ group: #DsMessages) "Category?"
			confirm: (self getMRIString: DtStringNotACategoryTryAgain group: #DsMessages)
				"'%1' is not the name of a method category.\\Try again?" 
			answer: ''  "$NON-NLS$"
			convertUsing: [:catName |
				self findCategoriesInSpace: anObjectSpace using: catName ifCancel: [self]].
		(theCategories isNil or: [theCategories == self]) ifFalse: [
			| methods |
			methods := (self dsInSpace: anObjectSpace) allMethodsInCategories: theCategories.
			methods isEmpty
				ifTrue: [
					theCategories size = 1
						ifTrue: [(self getMRI: DtMsgNoMethodsInCategory group: #DsMessages) displayReplace: (Array with: theCategories)
							"There are no methods in the category '%1'."]
						ifFalse: [(self getMRI: DtMsgNoMethodsInCategories group: #DsMessages) displayReplace: (Array with: theCategories asArray printString)
							"There are no methods in the categories %1."]]
				ifFalse: [
					| label |
					label := theCategories size = 1
						ifTrue: [(self getMRIString: DtTitleCategoryBrowserFor group: #DsMessages) bindWith: theCategories first
							"Category Browser for '%1'"]
						ifFalse: [(self getMRIString: DtTitleCategoryBrowser group: #DsMessages) 
							"Category Browser"].
					((DtDistributedUI remoteBrowser: #methods) 
							on: methods labeled: label) 
						associateObjectSpace: anObjectSpace;
						open]]]!

references: anObjectSpace
	"Open a browser on all the references to the variable provided by the user."

	(DtDistributedUI remoteBrowser: #highlightingMethods) == nil ifTrue: [^self].
	System showBusyCursorWhile: [
		| aString variableName methods |
		aString := System prompt: (self getMRIString: DtStringReferencesTo group: #DsMessages)
			"References to?".
		(aString isNil or: [aString isEmpty]) ifFalse: [
			variableName := self findVariableInSpace: anObjectSpace using: aString ifCancel: [self].
			variableName == self ifFalse: [
				variableName isNil
					ifTrue: [
						(self getMRI: DtMsgNoVariablesMatch group: #DsMessages) displayReplace: (Array with: aString)
							"No variables match '%1'"]
					ifFalse: [
						methods := (self dsInSpace: anObjectSpace) allMethodsReferencingLiteral: variableName.
						methods isEmpty
							ifTrue: [(self getMRI: DtMsgNoMethodsFound group: #DsMessages) display
								"No methods found"]
							ifFalse: [
								((DtDistributedUI remoteBrowser: #highlightingMethods)	
									on: methods
										labeled: ((self getMRIString: DtTitleReferencesTo group: #DsMessages) bindWith: variableName)
											"References to %1"
										highlighting: variableName)  
									associateObjectSpace: anObjectSpace;
									open
								]
						]	
				]
			]
		]!

implementors: anObjectSpace
		"Open a methods browser on the implementors of a message"

	(DtDistributedUI remoteBrowser: #methods) == nil ifTrue: [^self].
	System showBusyCursorWhile: [
		| methodName selector methods |
		methodName := System prompt: (self getMRIString: DtStringImplementorsOf group: #DsMessages).
			"Implementors of?"
		(methodName isNil or: [methodName isEmpty]) ifFalse: [
			selector :=  self findSelectorInSpace: anObjectSpace using: methodName ifCancel: [self]. 
			selector == self ifFalse: [
				selector == nil
					ifTrue: [
						(self getMRI: DtMsgNoSymbolsMatch group: #DsMessages) 
							displayReplace: (Array with: methodName)]
					ifFalse: [
						methods := (self dsInSpace: anObjectSpace) allMethodsNamed: selector.
						methods isEmpty
							ifTrue: [(self getMRI: DtMsgNoMethodsFound group: #DsMessages) display
								"No methods found."]
							ifFalse: [
								((DtDistributedUI remoteBrowser: #methods) 
									on: methods 
										labeled: ((self getMRIString: DtTitleImplementorsOf group: #DsMessages) bindWith: selector printString)
											"Implementors of %1"
										highlighting: selector) 		
									associateObjectSpace: anObjectSpace;
									open.
							]
						]
				]
			]
		]!

openApplicationBrowser: anObjectSpace
	"Open a browser on an application provided by the user."

	(DtDistributedUI remoteBrowser: #application) == nil ifTrue: [^self].
	System showBusyCursorWhile: [
		| theApp |
		theApp := self
			prompt: (self getMRIString: DtStringApplicationQ group: #DsMessages) "Application?"
			confirm: (self getMRIString: DtStringNotAnAppTryAgain group: #DsMessages)
				"'%1' is not the name of an application.\\Try again?" 
			answer: ''  "$NON-NLS$"
			convertUsing: [:appName | self findApplicationInSpace: anObjectSpace using: appName ifCancel: [self]].
		(theApp isNil or: [theApp == self]) ifFalse: [
			((DtDistributedUI remoteBrowser: #application) on: theApp)
				associateObjectSpace: anObjectSpace;
				open.
			]]!

openClassBrowser: anObjectSpace
	"Open a browser on the class provided by the user."

	(DtDistributedUI remoteBrowser: #class) == nil ifTrue: [^self].
	System showBusyCursorWhile: [
		| theClass |
		theClass := self
			prompt: (self getMRIString: DtStringClassQ group: #DsMessages) "Class?"
			confirm: (self getMRIString: DtStringNotAClassTryAgain group: #DsMessages) 
				"'%1' is not the name of a class.\\Try again?" 
			answer: ''  "$NON-NLS$"
			convertUsing: [:className | 
				self findClassInSpace: anObjectSpace using: className ifCancel: [self]].
		(theClass isNil or: [theClass == self]) ifFalse: [
			((DtDistributedUI remoteBrowser: #class) on: theClass in: theClass controller)
				associateObjectSpace: anObjectSpace;
				open.
			]]! !

!DtDistributedUI class privateMethods !

findApplicationInSpace: anObjectSpace using: aString ifCancel: exitBlock
	"Private - Return the application which matches aString
	(or part of aString). If the user cancelled after seeing
	the choices, evaluate the exitBlock."

	| result choice |
	result := (self dsInSpace: anObjectSpace) remoteFindApplicationUsing: aString ifCancel: exitBlock.
	"Check for either nil or an Application"
	result size == 0 ifTrue: [^result].
	"Must be a Collection"
	result := result dsInLocalSpace.
	choice := CwListPrompter new
		parent: CwWMShell focusShell;
		messageString:  (self getMRIString: DtStringChooseApplication group: #DsMessages);
			"Choose the appropriate application."
		items: result;
		selectedItems: #();
		printBlock: [:s | s];
		prompt.
	^choice isNil
		ifTrue: [exitBlock value]
		ifFalse: [anObjectSpace globalAt: choice]!

findCategoriesInSpace: anObjectSpace using: aString ifCancel: exitBlock
	"Private - Return the category which matches aString
	(or part of aString). If the user cancelled after seeing
	the choices, evaluate the exitBlock."

	| choices chosen |
	choices := (self dsInSpace: anObjectSpace) remoteFindCategoriesUsing: aString ifCancel: exitBlock.
	"Check for nil and collection smaller than 2."
	choices size < 2 ifTrue: [^choices].
	choices := choices dsInLocalSpace.
	chosen := CwListPrompter new
		parent: CwWMShell focusShell;
		selectionPolicy: (CwConstants at: #XmEXTENDEDSELECT asPoolKey ifAbsent: []);
		messageString: (self getMRIString: DtStringSelectTheCatebories group: #DsMessages);
			"Select the categories."
		items: choices asSortedCollection;
		selectedItems: #();
		printBlock: [:s | s];
		prompt.
	^chosen isNil
		ifTrue: [exitBlock value]
		ifFalse: [OrderedCollection new addAll: chosen; yourself]!

findClassInSpace: anObjectSpace using: aString ifCancel: exitBlock
	"Private - Return the class which matches aString
	(or part of aString). If the user cancelled after seeing
	the choices, evaluate the exitBlock."

	| result choice |
	result := (self dsInSpace: anObjectSpace) remoteFindClassUsing: aString ifCancel: exitBlock.
	"Check for nil or a class."
	result size == 0 ifTrue: [^result].
	"Must be a collection."
	result := result dsInLocalSpace.
	choice := CwListPrompter new
		parent: CwWMShell focusShell;
		messageString: (self getMRIString: DtStringChooseClass group: #DsMessages);
			"Choose the appropriate class."
		items: result;
		selectedItems: #();
		printBlock: [:s | s];
		prompt.
	^choice isNil
		ifTrue: [exitBlock value]
		ifFalse: [anObjectSpace globalAt: choice]!

findSelectorInSpace: anObjectSpace using: aString ifCancel: exitBlock
	"Private - Return the selector which matches aString
	(or part of aString) or nil if no choices exist. If the user
	cancelled after seeing the choices, evaluate the exitBlock."

	| result choice |
	result := (self dsInSpace: anObjectSpace) remoteFindSelectorUsing: aString ifCancel: exitBlock.
	result == nil ifTrue: [^nil].
	result isSymbol ifTrue: [^result].
	result := result dsInLocalSpace.
	choice := CwListPrompter new
		parent: CwWMShell focusShell;
		messageString: ((self getMRIString: DtStringChooseSelectorFor group: #DsMessages) bindWith: aString);
			"Choose the appropriate selector for '%1'"
		items: result asSortedCollection;
		selectedItems: (result size = 1 ifTrue: [result asArray] ifFalse: [#()]);
		printBlock: [:s | s];
		prompt.
	^choice isNil
		ifTrue: [exitBlock value] "cancelled"
		ifFalse: [choice]!

findSelectorsInSpace: anObjectSpace using: methodNames
		"Answer an array of selectors matching the strings
		in 'methodNames'. "

	| selectors |
	selectors :=  Set new.
	methodNames subStrings asSet do: [:methodName |
		| selector |
		selector := self findSelectorInSpace: anObjectSpace using: methodName ifCancel: [^Array new].
		selector == nil
			ifTrue: [
				(self dsGetMRI: DtMsgNoSymbolsMatch group: #DsMessages)
					displayReplace: (Array with: methodName)
					"No symbols match '%1'"]
			ifFalse: [ selectors add: selector]
		].
	^selectors asArray!

findVariableInSpace: anObjectSpace using: aString ifCancel: exitBlock
	"Private - Return the variable name which matches aString
	(or part of aString) or nil if no choices exist. If the user
	cancelled after seeing the choices, evaluate the exitBlock.
	NOTE: Do not check to see if its a valid variable name since
	it may be used to find deleted variables which are still
	referenced by existing methods."

	| result choice |
	result := (self dsInSpace: anObjectSpace) remoteFindVariableUsing: aString ifCancel: exitBlock.
	result == nil ifTrue: [^nil].
	result isString ifTrue: [^nil].
	result := result dsInLocalSpace.
	choice := CwListPrompter new
		parent: CwWMShell focusShell;
		messageString: ((self getMRIString: DtStringChooseVariableFor group: #DsMessages) bindWith: aString);
			"Choose the appropriate variable for '%1'"
		items: result asSortedCollection;
		selectedItems: (result size = 1 ifTrue: [result asArray] ifFalse: [#()]);
		printBlock: [:s | s];
		prompt.
	^choice isNil
		ifTrue: [exitBlock value] "cancelled"
		ifFalse: [choice asString]!

remoteFindApplicationUsing: aString ifCancel: exitBlock
	"Private - Return the application which matches aString
	(or part of aString). If the user cancelled after seeing
	the choices, evaluate the exitBlock."

	| result |
	((aString includes: $*) or: [aString includes: $#]) ifFalse: [
		result := Smalltalk classAt: aString asSymbol.
		(result notNil and: [SubApplication currentlyLoaded includes: result])
			ifTrue: [^result]
			ifFalse: [^self remoteFindApplicationUsing: aString, '*' ifCancel: exitBlock]].
	result := SortedCollection new.
	SubApplication currentlyLoaded do: [:app |
		(aString match: app name) ifTrue: [result add: app name]].
	result isEmpty ifTrue: [^nil].
	result size = 1 ifTrue: [^Smalltalk classAt: result first].
	^result!

remoteFindCategoriesUsing: aString ifCancel: exitBlock
	"Private - Return the category which matches aString
	(or part of aString). If the user cancelled after seeing
	the choices, evaluate the exitBlock."

	| allCategories testString choices chosen |
	allCategories := Set new.
	System classHierarchyRoots do: [:root |
		root withAllSubclassesDo: [:cl |
			allCategories addAll: cl methodCategories]].
	allCategories := allCategories asArray.
	testString := aString.
	((aString includes: $*) or: [aString includes: $#]) ifFalse: [
		(allCategories includes: aString)
			ifTrue: [^Array with: aString]
			ifFalse: [testString :=  aString, '*']].
	choices := #().
	choices := allCategories select: [:existing | testString match: existing].
	choices isEmpty ifTrue: [^nil].
	choices size = 1 ifTrue: [^choices asArray].
	^choices!

remoteFindClassUsing: aString ifCancel: exitBlock
	"Private - Return the class which matches aString
	(or part of aString). If the user cancelled after seeing
	the choices, evaluate the exitBlock."

	| result |
	((aString includes: $*) or: [aString includes: $#]) ifFalse: [
		result := Smalltalk at: aString asSymbol ifAbsent: [
			^self remoteFindClassUsing: aString, '*' ifCancel: exitBlock].
		result isBehavior
			ifTrue: [^result]
			ifFalse: [^result class]].
	System showBusyCursorWhile: [
		result := SortedCollection new.
		Smalltalk associationsDo: [:assoc |
			(assoc value isBehavior and: [aString match: assoc key]) ifTrue: [
				result add: assoc key]]].
	result isEmpty ifTrue: [^nil].
	result size = 1 ifTrue: [^Smalltalk classAt: result first].
	^result!

remoteFindSelectorUsing: aString ifCancel: exitBlock
	"Private - Return the selector which matches aString
	(or part of aString) or nil if no choices exist. If the user
	cancelled after seeing the choices, evaluate the exitBlock."

	| result |
	((aString includes: $*) or: [aString includes: $#]) ifFalse: [
		(Symbol symbolTable includes: aString)
			ifTrue: [^aString asSymbol]
			ifFalse: [^nil]].
	aString = '*' ifTrue: [^#*].	"Treat '*' as the multiply selector."

	result:= OrderedCollection new.
	System showBusyCursorWhile: [
		Symbol symbolTable do: [:sym |
			(sym size > 0 and: [(sym at: 1) isLowercase and: [aString match: sym]]) ifTrue: [
				result add: sym]]].
	result isEmpty ifTrue: [^nil].
	^result!

remoteFindVariableUsing: aString ifCancel: exitBlock
	"Private - Return the variable name which matches aString
	(or part of aString) or nil if no choices exist. If the user
	cancelled after seeing the choices, evaluate the exitBlock.
	NOTE: Do not check to see if its a valid variable name since
	it may be used to find deleted variables which are still
	referenced by existing methods."

	| result choice |
	((aString includes: $*) or: [aString includes: $#]) ifFalse: [^aString].
	result:= OrderedCollection new.
	System showBusyCursorWhile: [
		| allPools allVariables |
		allPools := Set new.
		allVariables := Smalltalk keys asOrderedCollection.
		System allClasses do: [:cl |
			allVariables addAll: cl classVarNames.
			allPools addAll: cl sharedPoolNames].
		 allPools do: [:poolName |
			allVariables addAll:
				(Smalltalk poolDictionaryAt: poolName ifAbsent: [Dictionary new]) keys].
		allVariables do: [:varName |
			(aString match: varName) ifTrue: [
				result add: varName]]].
	result isEmpty ifTrue: [^nil].
	^result! !

!DtRemoteTranscript publicMethods !

associateObjectSpace: anObjectSpace
		"After the receiver is associated with the name space
		add the receiver as a dependent of the Transcript in
		that name space."
	| existingRT result |
	existingRT := InstanceDictionary at: anObjectSpace ifAbsentPut: [self].
	existingRT == self ifFalse: [
		existingRT shell isDestroyed ifTrue: [
			existingRT := InstanceDictionary at: anObjectSpace put: self].
		].
	existingRT == self ifTrue: [
		"Don't already have one"
		^super associateObjectSpace: anObjectSpace.
		].
	"There is already a Remote Transcript open for this object space"
	^self become: existingRT.! !
